double cquad_dqng(double f(), double a, double b, double epsabs, double epsrel, double *abserr, int *neval, int *ier)
{
/******************************************************************************
***  Date written:         03/30/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/01/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@dma.unina.it)            ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***-------------------------------------------------------------------------***
***  Purpose:                                                               ***
***                                                                         ***
***    The function provides a C interface to the fortran routine dqng      ***
***    of quadpack package.                                                 ***
***                                                                         ***
***    The function calculates an approximation result to a given definite  ***
***    integral i = integral of f over (a,b), hopefully satisfying          ***
***    following claim for accuracy:                                        ***
***                                                                         ***
***              abs(i - result) <= max(epsabs, epsrel * abs(i))            ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    A simple non-adaptive automatic integrator, based on a sequence of   ***
***    rules with increasing degree of algebric precision.                  ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    a        lower limit of integration                                  ***
***                                                                         ***
***    b        upper limit of integration                                  ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***                                                                         ***
***    epsrel   relative accoracy requested                                 ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***               ier = 1 --> the maximum number of steps has been          ***
***                           executed. The integral is probably too        ***
***                           difficult to be calculated by dqng.           ***
***                                                                         ***
***               ier = 6 --> the input is invalid because epsabs <= 0 and  ***
***                           epsrel < max (50 * rel.mach.acc., 0.5d-28).   ***
***                           res, abserr and neval are set to zero.        ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/
    
    double res;

    dqng_(f, &a, &b, &epsabs, &epsrel, &res, abserr, neval, ier);

    return res;
}
